#!/usr/bin/python
import socket
from string import atoi
from sys import exit

object2win={}
# object to parent window retrieval dictionary
objectType={}
# object type retrieval dictionary
objectArray={}
# object retrieval dictionary, including windows

context={}
#The context passed to user-defined functions

sock=socket.socket(socket.AF_UNIX,socket.SOCK_STREAM)

def enable(IPCpath):
	"Starts the connection. Argument IPCPath is the path to the IPC socket."
	sock.connect(IPCpath)

def standby():
	"The main routine, really. Reads input from the socket and parses it."
	line = sock.recv(256)
	if line.startswith('QUIT'):
		print("Au revoir...")
		return("QUIT")

	if line.startswith('CL,'):
	# This is a CLick event, most possibly on a pushbutton
	# (haven't implemented too many controls :-)
	# Let's retrieve the object and call its Action method
		print(objectArray)
		rfs=line.split(",")
		rf=rfs[1]
		print("Click on GUI control [%s] %s"%(objectType[rf],rf))
		obj=objectArray[rf]
		obj.Action()
		return "Object reference: [%s] ==> %s"%(rf,objectType[rf])

	if line.startswith('RS,'):
	# A window has been resized
	# Let's retrieve the data
	# RS,winRef,width,heigh,top,left
		data=line.split(",")
		print(line)
		rf=data[1]
		obj=objectArray[rf]
		#obj is the window
		obj.width=atoi(data[2])
		obj.height=atoi(data[3])
		obj.top=atoi(data[4])
		obj.left=atoi(data[5])
		obj.resized()
		return "Resized the window."


	return line

class RBWindow:
	"""Creates a Document Window on the RB Framework
	Params are width, height, top, left, Title, [visible=false]"""
	
	def __init__(self, w=640, h=480, tp=-1, lf=-1, tt="Untitled", vs="False"):
		self.width=w
		self.height=h
		self.top=tp
		self.left=lf
		# Defaults to -1, which will be rewritten as center,center in the RB framework
		self.title=tt
		self.visible=vs
		sock.send("NW,%i,%i,%i,%i,%s,%s\n"%(w,h,tp,lf,tt,vs))
		self.ref=sock.recv(32)
		objectArray[self.ref]=self
		print("%s's winRef is '%s'"%(self.title,self.ref))


	def close(self):
		"Sends a CloseWindow message to the framework"
		sock.send("CW,%s\n"%self.ref)

	def show(self):
		"The visibility of windows defaults to False. This method brings up the window (and the framework to the front)."
		sock.send("SH,%s\n"%self.ref)

	def addPushButton(self,pb):
		"Asks the RB framework to attach a previously created button to the current window"
		sock.send("NE,PB,%i,%i,%i,%i,%s,%s,%i,%s\n"%(pb.width, pb.height, pb.top, pb.left, pb.caption, pb.font, pb.fontsize,self.ref))
		pb.ref=sock.recv(32)
		object2win[pb.ref]=self.ref
		objectType[pb.ref]="PB"
		objectArray[pb.ref]=pb
		print("%s's Ref is '%s'"%(pb.caption,pb.ref))

	def addEditField(self,ef):
		"Asks the RB framework to attach a previously created editField to the current window."
		sock.send("NE,EF,%i,%i,%i,%i,%s,%s,%i,%s\n"%(ef.width, ef.height, ef.top, ef.left, ef.defaulttext, ef.font, ef.fontsize,self.ref))
		ef.ref=sock.recv(32)
		object2win[ef.ref]=self.ref
		objectType[ef.ref]="EF"
		objectArray[ef.ref]=ef

	def addListBox(self,lb):
		"Asks the RB framework to attach a previously created ListBox to the current window."
		sock.send("NE,LB,%i,%i,%i,%i,%i,%s,%s,%i,%s\n"%(lb.width, lb.height, lb.top, lb.left, lb.columnCount, lb.columnWidths, lb.font, lb.fontsize,self.ref))
		lb.ref=sock.recv(32)
		object2win[lb.ref]=self.ref
		objectType[lb.ref]="LB"
		objectArray[lb.ref]=lb

	def resized(self): #self-explanatory I hope
		"A placeholder for the .Resized event."
		pass


class RBPushButton:
	"""Creates a push button that can then be added to a window.
	Just creating it won't do anything until you attach it to a window."""
	
	def __init__(self,w=-1,h=-1,tp=-1,lf=-1,cp="Untitled",ft="",fs=0):
		self.width=w
		self.height=h
		self.top=tp
		self.left=lf
		# Defaults to -1, which will be rewritten as center,center in the RB framework
		self.caption=cp
		self.font=ft
		self.fontsize=fs
		self.ref=""
		print("Instance of RBPushButton [%s] created"%self.caption)


class RBEditField:
	"""Creates an editfield that can then be added to a window.
	Just creating it won't do anything until you attach it to a window."""
	
	def __init__(self,w=-1,h=-1,tp=-1,lf=-1,txt="",ft="",fs=0):
		self.width=w
		self.height=h
		self.top=tp
		self.left=lf
		# Defaults to -1, which will be rewritten as center,center in the RB framework
		self.defaulttext=txt
		self.font=ft
		self.fontsize=fs
		self.ref=""

	def text(self):
		"Returns the contents of the EditField."
		wr=object2win[self.ref]
		efr=self.ref
		sock.send("PR,%s,%s,TX,GT\n"%(wr,efr))
		
		return sock.recv(1024)

class RBListBox:
	"""Creates a ListBox control that can then be added to a window.
	Just creating it won't do anything until you attach it to a window."""
	
	def __init__(self,w=-1,h=-1,tp=-1,lf=-1,cc=1,cw="",ft="",fs=0):
		self.width=w
		self.height=h
		# Defaults to -1, which will be rewritten as w-40, h-28 in the RB framework
		self.top=tp
		self.left=lf
		# Defaults to -1, which will be rewritten as top,left [14,20] in the RB framework
		self.columnCount=cc
		self.columnWidths=cw
		self.font=ft
		self.fontsize=fs
		self.ref=""

	def populate(self,pp):
		"""Asks the RB framework to populate a ListBox with data formatted as follows:
		rows are delimited by commas, columns by semi-colons. Actual commas and semi-colons
		have to be escaped with \\"""
		wr=object2win[self.ref]
		lbr=self.ref
		sock.send("PP,%s,%s,%s\n"%(wr,lbr,pp))

	def cell(self,rw,cl):
		"Returns the contents of a Cell."
		wr=object2win[self.ref]
		lbr=self.ref
		sock.send("PR,%s,%s,CL,GT,%s,%s\n"%(wr,lbr,rw,cl))
		return sock.recv(256)
		
	def listIndex(self):
		"Returns the selected row."
		wr=object2win[self.ref]
		lbr=self.ref
		sock.send("PR,%s,%s,LI,GT\n"%(wr,lbr))
		return sock.recv(32)
